/********************************************************
 Name          : gpio_int_pwm.c
 Author        : adp@unipi.gr
 Description   : ---LED3  , 
 		 ---PB0  LED2 
 		         LED6,
 		 ---PB1  LED4 
 		         LED6.
 **********************************************************/
#include "compiler.h"
#include "intc.h"
#include "gpio.h"
#include "pwm.h"
#include "board.h"

unsigned int channel_id;

//      PB0 
// (     LED6)
#if __GNUC__
	__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif

static void PB0_int_handler(void)
{		
volatile avr32_pwm_t *pwm = &AVR32_PWM;
extern unsigned int channel_id;
int intensity;

if (gpio_get_pin_value(GPIO_PUSH_BUTTON_0) == 1)
	gpio_set_gpio_pin(LED1_GPIO);
else
	{
	gpio_clr_gpio_pin(LED1_GPIO);	
	intensity=pwm->channel[channel_id].cdty;
	if (intensity>0) 
		intensity--;
	pwm->channel[channel_id].cupd=intensity;
	}
gpio_clear_pin_interrupt_flag(GPIO_PUSH_BUTTON_0);
}

//      PB1 
// (     LED6)	
#if __GNUC__
	__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif	

static void PB1_int_handler(void)
{
volatile avr32_pwm_t *pwm = &AVR32_PWM;		
extern unsigned int channel_id;
int intensity;

if (gpio_get_pin_value(GPIO_PUSH_BUTTON_1) == 1)
    gpio_set_gpio_pin(LED3_GPIO);
else
    {
    gpio_clr_gpio_pin(LED3_GPIO);
    intensity=pwm->channel[channel_id].cdty;
    if (intensity<pwm->channel[channel_id].cprd-1) 
    	intensity++;
    pwm->channel[channel_id].cupd=intensity;
    }
    
gpio_clear_pin_interrupt_flag(GPIO_PUSH_BUTTON_1);
}		 

//  	
int main(void)
{
U32 i,r;
pwm_opt_t pwm_opt;        
avr32_pwm_channel_t pwm_channel;		
extern unsigned int channel_id;
	 
	  
Disable_global_interrupt();
INTC_init_interrupts();
		
//   2  PWM   GPIO pin 53 (PB21-LED6 )
gpio_enable_module_pin(AVR32_PWM_2_PIN, AVR32_PWM_2_FUNCTION);
	  
//  PWM MR
//  PWM      
pwm_opt.diva = AVR32_PWM_DIVA_CLK_OFF;
pwm_opt.divb = AVR32_PWM_DIVB_CLK_OFF;
pwm_opt.prea = AVR32_PWM_PREA_MCK;
pwm_opt.preb = AVR32_PWM_PREB_MCK;
pwm_init(&pwm_opt);
	  
//   PWM 2
channel_id = 2;
pwm_channel.CMR.calg = PWM_MODE_LEFT_ALIGNED;       //  
pwm_channel.CMR.cpol = PWM_POLARITY_LOW;            //  
pwm_channel.CMR.cpd = PWM_UPDATE_DUTY;              //  duty cycle
pwm_channel.CMR.cpre = AVR32_PWM_CPRE_MCK_DIV_64;  //   115200/256 Hz
pwm_channel.cdty = 19;  // duty cycle = 1-19/20
pwm_channel.cprd = 20;  //  20*256/115200 ~= 44msecs
pwm_channel.cupd = 0;   // Channel update is not used here.
pwm_channel_init(channel_id, &pwm_channel); // Set channel configuration to channel 0.
pwm_start_channels(1 << channel_id);  // Start channel 3.

//    PB0  PB1
INTC_register_interrupt(&PB0_int_handler,AVR32_GPIO_IRQ_11, AVR32_INTC_INT0);     
INTC_register_interrupt(&PB1_int_handler,AVR32_GPIO_IRQ_10, AVR32_INTC_INT0);  
gpio_enable_pin_glitch_filter(GPIO_PUSH_BUTTON_0);
gpio_enable_pin_interrupt(GPIO_PUSH_BUTTON_0,GPIO_PIN_CHANGE);
gpio_enable_pin_glitch_filter(GPIO_PUSH_BUTTON_1);
gpio_enable_pin_interrupt(GPIO_PUSH_BUTTON_1,GPIO_PIN_CHANGE);	
Enable_global_interrupt();

while (1)
	{
    for (i = 0; i < 1000; i += 1) 
    	r=rand();  
	gpio_tgl_gpio_pin(LED2_GPIO);
	}
}
